﻿Imports System.ComponentModel.Design
Imports System.ComponentModel.Design.Serialization
Imports System.CodeDom
Imports EventBindings.Extensions

Namespace Design
    Public Class EventBindingSourceSerializer
        Inherits CodeDomSerializer

        Public Overrides Function Serialize(ByVal manager As IDesignerSerializationManager, ByVal value As Object) As Object
            Dim baseClassSerializer As CodeDomSerializer = CType(manager.GetSerializer(value.GetType.BaseType, GetType(CodeDomSerializer)), CodeDomSerializer)
            Dim codeObject As Object = baseClassSerializer.Serialize(manager, value)

            If Not TypeOf codeObject Is CodeStatementCollection Then Return codeObject
            If Not TypeOf value Is IEventBindingSource Then Return codeObject

            Dim BindingSource = DirectCast(value, IEventBindingSource)
            Dim Bindings = BindingSource.GetBindableBindings
            If Bindings.IsNullOrEmpty Then Return codeObject

            Dim statements = DirectCast(codeObject, CodeStatementCollection)
            Dim BindingSourceExpression = Me.GetSerializedItemExpression(manager, BindingSource)
            Dim BindingSourceName = Me.GetUniqueName(manager, BindingSource)

            If Not statements.ContainsNameComments(BindingSourceName) Then statements.AddNameComments(BindingSourceName)

            For Each Binding In Bindings
                Dim Method = GetAddBindingCallCode(BindingSourceExpression, manager, Binding)
                If Method Is Nothing Then Continue For
                statements.Add(Method)
            Next
            statements.Add(New CodeCommentStatement(""))
            Return codeObject

        End Function

        Private Function GetAddBindingCallCode(ByVal bindingSourceExpression As CodeExpression, ByVal manager As IDesignerSerializationManager, ByVal binding As EventBinding) As CodeMethodInvokeExpression

            'Signature de la méthode:
            'Function Add(
            '   ByVal eventName As String, ByVal eventSource As Object,
            '   ByVal handlerID As Object, ByVal handlerProvider As IEventHandlerProvider, ByVal handlerSource As Object,
            '   ByVal mediationFactory As IEventMediatorFactory, ByVal mediationMode As MediationMode
            ') As EventBinding

            Try
                Dim CreateMethod As New CodeMethodInvokeExpression(bindingSourceExpression, "Add")
                'Event
                CreateMethod.Parameters.Add(Me.SerializeToExpression(manager, binding.Event.EventName))
                CreateMethod.Parameters.Add(Me.GetSerializedItemExpression(manager, binding.Event.EventSource))
                'Handling
                CreateMethod.Parameters.Add(Me.GetSerializedItemExpression(manager, binding.Handling.EventHandlerID))
                CreateMethod.Parameters.Add(Me.GetSerializedItemExpression(manager, binding.Handling.Provider))
                CreateMethod.Parameters.Add(Me.GetSerializedItemExpression(manager, binding.Handling.EventHandlerSource))
                'Mediation
                CreateMethod.Parameters.Add(Me.GetSerializedItemExpression(manager, binding.Mediation.Factory))
                CreateMethod.Parameters.Add(Me.SerializeToExpression(manager, binding.Mediation.MediationMode))
                Return CreateMethod
            Catch ex As Exception
            End Try
            Return Nothing
        End Function

        Private Function GetSerializedItemExpression(ByVal manager As IDesignerSerializationManager, ByVal item As Object) As CodeExpression
            Dim Expression As CodeExpression = Nothing
            If item IsNot Nothing AndAlso Me.IsSerialized(manager, item, True) Then
                Expression = Me.GetExpression(manager, item)
            Else
                Expression = Me.SerializeToExpression(manager, item)
            End If
            Return Expression
        End Function
    End Class
End Namespace

