﻿Imports System.ComponentModel.Design
Imports System.Windows.Forms.Design
Imports EventBindings.Extensions

Namespace Design
    <PermissionSetAttribute(SecurityAction.Demand, Name:="FullTrust")>
    Public Class EventBindingsPropertyTab
        Inherits PropertyTab

#Region "Fields"
        Private _ServiceProvider As IEventBindingServiceProvider
#End Region

#Region "Constructors"
        Private Sub New()
        End Sub
        Public Sub New(ByVal sp As IServiceProvider)
            MyBase.New()
            If sp Is Nothing Then Throw New ArgumentNullException("sp")
            Me._ServiceProvider = sp.GetOrCreateEventBindingServiceProvider
            Me._ServiceProvider.RetrieveServicesFromHostView()
        End Sub
#End Region

#Region "Properties"
        Private ReadOnly Property DiscoveryService As IEventBindingDiscoveryService
            Get
                Return Me._ServiceProvider.DiscoryService
            End Get
        End Property
#End Region

#Region "Overrides"
        Public Overrides ReadOnly Property Bitmap As System.Drawing.Bitmap
            Get
                Return My.Resources.EventBindingsPropertyTab
            End Get
        End Property
        Public Overrides ReadOnly Property TabName As String
            Get
                Return My.Resources.EventBindingsTabName
            End Get
        End Property
        Public Overrides ReadOnly Property HelpKeyword As String
            Get
                Return My.Resources.EventBindingsHelpKeyWord
            End Get
        End Property

        Public Overrides Function CanExtend(ByVal extendee As Object) As Boolean
            Return Me.DiscoveryService.HasEvents(extendee)
        End Function

        Public Overrides Function GetDefaultProperty(ByVal component As Object) As PropertyDescriptor
            Return Me.DiscoveryService.GetDefaultEventProperty(component)
        End Function

        Public Overloads Overrides Function GetProperties(ByVal component As Object, ByVal attributes() As Attribute) As PropertyDescriptorCollection
            Return Me.DiscoveryService.GetEventProperties(component, attributes)
        End Function
        Public Overrides Function GetProperties(ByVal component As Object) As PropertyDescriptorCollection
            Return Me.DiscoveryService.GetEventProperties(component)
        End Function
        Public Overrides Function GetProperties(ByVal context As ITypeDescriptorContext, ByVal component As Object, ByVal attributes() As Attribute) As PropertyDescriptorCollection
            If context.IsRootComponent(component) Then
                Return Me.DiscoveryService.GetEventProperties(component, attributes)
            ElseIf TypeOf component Is EventBinding() Then
                Dim Converter As New ArrayConverter
                Return Converter.GetProperties(context, component, attributes)
            Else
                Return TypeDescriptor.GetProperties(component, attributes)
            End If
        End Function
#End Region

    End Class

End Namespace
