﻿Imports System.ComponentModel.Design
Imports EventBindings.Extensions
Namespace Design
    Public Class MediationDataBox

#Region "Fields"
        Private _MediatorFactories As ComboBoxController(Of IEventMediatorFactory)
        Private _Data As MediationData
        Private _Binding As EventBinding
        Private _ChangeService As IComponentChangeService
#End Region

#Region "Events"
        Public Event DataChanged As EventHandler
#End Region

#Region "Constructor"
        Public Sub New()

            ' Cet appel est requis par le concepteur.
            InitializeComponent()

            ' Ajoutez une initialisation quelconque après l'appel InitializeComponent().
            Me._MediatorFactories = New ComboBoxController(Of IEventMediatorFactory)(Me.cbbMediatorFactory, AddressOf OnMediatorFactoryChanged)
            Me.ErrorProvider1.Icon = SystemIcons.Warning
        End Sub
#End Region

#Region "Methods"
        Public Sub RefreshWarning()
            Dim Warning As String = ""
            If Me._Data IsNot Nothing AndAlso Me._Data.Factory IsNot Nothing AndAlso Me._Binding IsNot Nothing Then
                Try
                    If Not Me._Data.Factory.SupportMediationFor(Me._Binding) Then
                        Warning = "Cette fabrique de médiateur ne supporte pas cette liaison!"
                    End If
                Catch ex As Exception
                    Warning = ex.Message
                End Try
            End If
            Me.ErrorProvider1.SetError(Me.FactoryLabel, Warning)
        End Sub
        Private Sub RaiseDataChanged()
            If Me._Binding IsNot Nothing AndAlso Me._Binding.Source IsNot Nothing AndAlso Me._ChangeService IsNot Nothing Then
                Me._ChangeService.OnComponentChanged(Me._Binding.Source, Nothing, Nothing, Nothing)
            End If
            RaiseEvent DataChanged(Me, EventArgs.Empty)
        End Sub
        Public Sub Fill(ByVal binding As EventBinding, ByVal provider As IEventBindingServiceProvider)
            If binding Is Nothing Then Throw New ArgumentNullException("binding")
            Me.Fill(binding.Mediation, provider)
            Me._Binding = binding
        End Sub
        Public Sub Fill(ByVal mediation As MediationData, ByVal provider As IEventBindingServiceProvider)
            If mediation Is Nothing Then Throw New ArgumentNullException("mediation")
            Me._ChangeService = provider.GetService(Of IComponentChangeService)()
            Me._Data = mediation
            Me._MediatorFactories.ReferenceService = provider.GetService(Of IReferenceService)()
            Me._MediatorFactories.Items = provider.MediatorFactories
            Me._MediatorFactories.SelectedItem = Me._Data.Factory
            Me.RemoveMediationHandlers()
            Me.rbSynchronous.Checked = (Me._Data.MediationMode = MediationMode.Synchronous)
            Me.rbAsynchronous.Checked = (Me._Data.MediationMode = MediationMode.Asynchronous)
            Me.rbSynchronized.Checked = (Me._Data.MediationMode = MediationMode.Synchronized)
            Me.AddMediationHandlers()
        End Sub
        Private Sub AddMediationHandlers()
            AddHandler Me.rbSynchronous.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
            AddHandler Me.rbSynchronized.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
            AddHandler Me.rbAsynchronous.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
        End Sub
        Private Sub RemoveMediationHandlers()
            RemoveHandler Me.rbSynchronous.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
            RemoveHandler Me.rbSynchronized.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
            RemoveHandler Me.rbAsynchronous.CheckedChanged, AddressOf MediationTypeRadionButton_CheckedChanged
        End Sub
#End Region

#Region "Event Handlers"
        Private Sub OnMediatorFactoryChanged(ByVal value As IEventMediatorFactory)
            Me._Data.Factory = value
            Me.RaiseDataChanged()
            Me.RefreshWarning()
        End Sub
        Private Sub MediationTypeRadionButton_CheckedChanged(ByVal sender As Object, ByVal e As EventArgs)
            If Me.rbAsynchronous.Checked Then
                Me._Data.MediationMode = MediationMode.Asynchronous
            ElseIf Me.rbSynchronized.Checked Then
                Me._Data.MediationMode = MediationMode.Synchronized
            Else
                Me._Data.MediationMode = MediationMode.Synchronous
            End If
            Me.RaiseDataChanged()
            Me.RefreshWarning()
        End Sub
#End Region

    End Class
End Namespace

