﻿Namespace Extensions
    <Extension()>
    Public Module ComponentModelExtensions

#Region "EventDescriptorCollection"
        <Extension()>
        Public Function GetEvent(ByVal COL As EventDescriptorCollection, ByVal eventName As String, ByVal stringComparison As StringComparison) As EventDescriptor
            For Each Descriptor As EventDescriptor In COL
                If Not String.Equals(Descriptor.Name, eventName, stringComparison) Then Continue For
                Return Descriptor
            Next
            Return Nothing
        End Function
#End Region

#Region "EventDescriptor"
        <Extension()>
        Public Function IsGenericEventHandler(ByVal ED As EventDescriptor) As Boolean
            If Not ED.EventType.IsGenericType Then Return False
            Dim GenDefinition = ED.EventType.GetGenericTypeDefinition
            Return GenDefinition Is GetType(EventHandler(Of ))
        End Function
        <Extension()>
        Public Function IsEventHandler(ByVal ED As EventDescriptor) As Boolean
            Return ED.EventType Is GetType(EventHandler)
        End Function
        <Extension()>
        Public Function GetCustomEventArgsType(ByVal ED As EventDescriptor) As Type
            Dim InvokeMethod = ED.EventType.GetMethod("Invoke")
            If InvokeMethod Is Nothing Then Return Nothing
            Dim Parameters = InvokeMethod.GetParameters
            If Parameters.IsNullOrEmpty Then Return Nothing
            For Each Parameter In Parameters
                If Parameter.ParameterType.IsType(Of EventArgs)() Then Return Parameter.ParameterType
            Next
            Return Nothing
        End Function
        <Extension()>
        Public Function IsCustomEventHandler(ByVal ED As EventDescriptor) As Boolean
            Dim InvokeMethod = ED.EventType.GetMethod("Invoke")
            If InvokeMethod Is Nothing Then Return False
            If InvokeMethod.ReturnType IsNot GetType(Void) Then Return False
            Dim Parameters = InvokeMethod.GetParameters
            If Parameters.IsNullOrEmpty Then Return False
            If Parameters.Count <> 2 Then Return False
            Return Parameters(0).ParameterType Is GetType(Object) AndAlso Parameters(1).ParameterType.IsType(Of EventArgs)()
        End Function
        <Extension()>
        Public Function GetEventArgsType(ByVal ED As EventDescriptor) As Type
            If ED Is Nothing Then Return Nothing
            If ED.IsEventHandler Then Return GetType(EventArgs)
            If ED.IsGenericEventHandler Then Return ED.EventType.GetGenericArguments(0)
            Return ED.GetCustomEventArgsType
        End Function
#End Region

#Region "ITypeDescriptorContext"
        <Extension()>
        Public Function IsRootComponent(ByVal context As ITypeDescriptorContext, ByVal component As Object) As Boolean
            Return context IsNot Nothing AndAlso context.Instance Is component
        End Function
#End Region

    End Module
End Namespace

