﻿Imports System.Reflection
Namespace Extensions
    <Extension()>
    Public Module ReflectionExtensions

#Region "MemberInfo"
        <Extension()>
        Public Function TryGetFirstAttribute(Of TAttribute As Attribute)(
                ByVal MI As MemberInfo, ByVal inherit As Boolean, ByRef attribute As TAttribute) As Boolean

            Dim Atts = MI.GetCustomAttributes(GetType(TAttribute), inherit)
            If Atts.IsNullOrEmpty Then Return False
            attribute = DirectCast(Atts.First, TAttribute)
            Return True
        End Function
        <Extension()>
        Public Function HasAttribute(Of TAttribute As Attribute)(
                ByVal MI As MemberInfo, ByVal inherit As Boolean) As Boolean
            Dim Atts = MI.GetCustomAttributes(GetType(TAttribute), inherit)
            Return Not Atts.IsNullOrEmpty
        End Function
        <Extension()>
        Public Function ReturnType(Of TReturn)(ByVal MI As MemberInfo) As Boolean
            If MI Is Nothing Then Return False
            If TypeOf MI Is FieldInfo Then
                Return DirectCast(MI, FieldInfo).FieldType.IsType(Of TReturn)()
            ElseIf TypeOf MI Is MethodInfo Then
                Return DirectCast(MI, MethodInfo).ReturnType.IsType(Of TReturn)()
            ElseIf TypeOf MI Is PropertyInfo Then
                Dim PI = DirectCast(MI, PropertyInfo)
                If Not PI.CanRead Then Return False
                Return PI.PropertyType.IsType(Of TReturn)()
            End If
            Return False
        End Function
        <Extension()>
        Public Function HasAnyKindOfParameters(ByVal MI As MemberInfo) As Boolean
            If MI Is Nothing Then Return False
            If TypeOf MI Is FieldInfo Then
                Return False
            ElseIf TypeOf MI Is MethodBase Then
                Return DirectCast(MI, MethodBase).HasParameters
            ElseIf TypeOf MI Is PropertyInfo Then
                Return DirectCast(MI, PropertyInfo).HasIndexParameters
            End If
            Return False
        End Function
#End Region
#Region "MethodBase"
        <Extension()>
        Public Function HasParameters(ByVal M As MethodBase) As Boolean
            If M Is Nothing Then Return False
            Return Not M.GetParameters.IsNullOrEmpty
        End Function
#End Region
#Region "PropertyInfo"
        <Extension()>
        Public Function HasIndexParameters(ByVal PI As PropertyInfo) As Boolean
            If PI Is Nothing Then Return False
            Return Not PI.GetIndexParameters.IsNullOrEmpty
        End Function
#End Region
#Region "Type"
        <Extension()> _
        Public Function IsType(Of TBaseType)(ByVal T As Type) As Boolean
            If T Is Nothing Then Return False
            Dim BaseType = GetType(TBaseType)
            If T Is BaseType Then Return True
            If T.IsSubclassOf(BaseType) Then Return True

            If BaseType.IsInterface Then
                Dim Interfaces = T.GetInterfaces
                If Interfaces.Contains(BaseType) Then Return True
            End If
            Return False
        End Function
        <Extension()> _
        Public Function IsTypeOf(ByVal T As Type, ByVal baseType As Type) As Boolean

            If baseType Is Nothing Then Throw New ArgumentNullException("baseType")
            If T Is Nothing Then Return False

            If T Is baseType Then Return True
            If T.IsSubclassOf(baseType) Then Return True

            If baseType.IsGenericTypeDefinition AndAlso T.IsGenericType Then
                If T.GetGenericTypeDefinition Is baseType Then Return True
            End If

            If baseType.IsInterface Then
                Dim Interfaces = T.GetInterfaces
                If Interfaces.Contains(baseType) Then Return True
            End If
            Return False
        End Function
#End Region
#Region "ParameterInfo"
        <Extension()>
        Public Function IsSenderParameter(ByVal PI As ParameterInfo) As Boolean
            If PI Is Nothing Then Return False
            If Not String.Equals(PI.Name, "sender", StringComparison.InvariantCultureIgnoreCase) Then Return False
            Return PI.ParameterType Is GetType(Object)
        End Function
        <Extension()>
        Public Function GetArgumentValue(ByVal PI As ParameterInfo, ByVal sender As Object, ByVal e As EventArgs) As Object
            If PI.IsSenderParameter Then
                Return sender
            ElseIf PI.ParameterType.IsAssignableFrom(e.GetType) Then
                Return e
            End If
            Return Nothing
        End Function
#End Region
    End Module
End Namespace
