﻿Imports System.ComponentModel.Design
Imports System.Drawing.Design
Imports EventBindings.Extensions

Public Class EventBindingDescriptor
    Inherits PropertyDescriptor

#Region "Fields"
    Private _Event As EventData
    Private _ServiceProvider As IEventBindingServiceProvider
#End Region

#Region "Constructor"
    Public Sub New(ByVal [event] As EventData, ByVal serviceProvider As IEventBindingServiceProvider)
        MyBase.New([event].EventDescriptor)
        If serviceProvider Is Nothing Then Throw New ArgumentNullException("serviceProvider")
        Me._Event = [event]
        Me._ServiceProvider = serviceProvider
    End Sub
#End Region

#Region "Properties"
    Public ReadOnly Property [Event] As EventData
        Get
            Return Me._Event
        End Get
    End Property
    Public ReadOnly Property ServiceProvider As IEventBindingServiceProvider
        Get
            Return Me._ServiceProvider
        End Get
    End Property
    Public Overrides ReadOnly Property ComponentType As Type
        Get
            Return Me._Event.EventDescriptor.ComponentType
        End Get
    End Property
    Public Overrides ReadOnly Property IsReadOnly As Boolean
        Get
            Return False
        End Get
    End Property
    Public Overrides ReadOnly Property PropertyType As Type
        Get
            Return GetType(EventBinding())
        End Get
    End Property
    Public Overrides ReadOnly Property Converter As TypeConverter
        Get
            Return New EventPropertyConverter
        End Get
    End Property
#End Region

#Region "Methods"
    Private Function GetDefaultValue() As Object
        Return New EventBinding() {New EventBinding(Me._Event)}
    End Function
    Public Overrides Function GetEditor(ByVal editorBaseType As Type) As Object
        Return New Design.EventBindingsTypeEditor
    End Function
    Public Overrides Function CanResetValue(ByVal component As Object) As Boolean
        Me.CheckComponentIsMine(component)
        Return Me.GetValue(component) IsNot Nothing
    End Function
    Public Overrides Function GetValue(ByVal component As Object) As Object
        Me.CheckComponentIsMine(component)
        Dim Bindings = Me._ServiceProvider.GetBindings(Me._Event)
        If Bindings.IsNullOrEmpty Then Return Me.GetDefaultValue
        Return Bindings
    End Function
    Public Overrides Sub ResetValue(ByVal component As Object)
        Me.CheckComponentIsMine(component)
        Me._ServiceProvider.ResetBindings(Me._Event)
    End Sub
    Public Overrides Sub SetValue(ByVal component As Object, ByVal value As Object)
        Me.CheckComponentIsMine(component)
        'Me._ServiceProvider.DescriptionService.SetValue(Me._EventDescriptor, component, value)
    End Sub
    Public Overrides Function ShouldSerializeValue(ByVal component As Object) As Boolean
        Me.CheckComponentIsMine(component)
        Return Me.GetValue(component) IsNot Nothing
    End Function
#End Region

#Region "Methods"
    Private Sub CheckComponentIsMine(ByVal component As Object)
        If Me._Event.EventSource Is component Then Exit Sub
        Throw New NotSupportedException("Bad component")
    End Sub
#End Region

End Class
