﻿Imports System.ComponentModel.Design
Imports System.ComponentModel.Design.Serialization

<DesignerSerializer(GetType(Design.EventBindingSourceSerializer), GetType(CodeDomSerializer))>
Public Class EventBindingSource
    Implements IEventBindingSource

#Region "Fields"
    Private _EventBindings As New Dictionary(Of EventData, List(Of EventBinding))
    Private _Bindings As New List(Of EventBinding)
#End Region

#Region "Methods"
    Public Function Add(ByVal eventName As String, ByVal eventSource As Object,
                        ByVal handlerID As Object, ByVal handlerProvider As IEventHandlerProvider, ByVal handlerSource As Object,
                        ByVal mediationFactory As IEventMediatorFactory, ByVal mediationMode As MediationMode
                        ) As EventBinding Implements IEventBindingSource.Add
        Dim ToAdd As New EventBinding(eventName, eventSource)
        ToAdd.Handling = New HandlingData With {
            .Provider = handlerProvider, .EventHandlerSource = handlerSource, .EventHandlerID = handlerID}
        ToAdd.Mediation = New MediationData(mediationFactory, mediationMode)
        ToAdd.Source = Me
        Return ToAdd
    End Function
    Public Sub Add(ByVal binding As EventBinding) Implements IEventBindingSource.Add
        If binding Is Nothing Then Throw New ArgumentNullException("binding")
        If Me._Bindings.Contains(binding) Then Exit Sub
        Me._Bindings.Add(binding)
        Dim Index As List(Of EventBinding) = Nothing
        If Not Me._EventBindings.TryGetValue(binding.Event, Index) Then
            Index = New List(Of EventBinding)
            Me._EventBindings.Add(binding.Event, Index)
        End If
        Index.Add(binding)
    End Sub
    Public Sub Remove(ByVal binding As EventBinding) Implements IEventBindingSource.Remove
        If binding Is Nothing Then Throw New ArgumentNullException("binding")
        Dim Index As List(Of EventBinding) = Nothing
        If Me._EventBindings.TryGetValue(binding.Event, Index) Then Index.Remove(binding)
        Me._Bindings.Remove(binding)
    End Sub

    Public Sub ResetBindings(ByVal [event] As EventData) Implements IEventBindingSource.ResetBindings
        Dim Index As List(Of EventBinding) = Nothing
        If Not Me._EventBindings.TryGetValue([event], Index) Then Exit Sub
        For Each item In Index
            Me._Bindings.Remove(item)
        Next
        Index.Clear()
        Me._EventBindings.Remove([event])
    End Sub
    Public Function GetBindings(ByVal [event] As EventData) As IEnumerable(Of EventBinding) Implements IEventBindingSource.GetBindings
        Dim Index As List(Of EventBinding) = Nothing
        If Not Me._EventBindings.TryGetValue([event], Index) Then Return Nothing
        Return Index.ToArray
    End Function
    Public Function GetBindings() As IEnumerable(Of EventBinding) Implements IEventBindingSource.GetBindings
        Return Me._Bindings.ToArray
    End Function
#End Region

End Class
