﻿Public Class EventsBinder
    Implements IEventBinder
    Implements ISupportInitialize

#Region "Fields & Events"
    Private _Handled As Boolean = False
    Public Event HandledChanged(ByVal sender As Object, ByVal e As EventArgs) Implements IEventBinder.HandledChanged
#End Region

#Region "Properties"
    Public Property BindingMode As EventBindingMode
    Public Property BindingSource As IEventBindingSource
    <DefaultValue(CStr(Nothing))>
    <AttributeProvider(GetType(IListSource))>
    Public Property DataSource As Object
    Public ReadOnly Property Handled As Boolean Implements IEventBinder.Handled
        Get
            Return Me._Handled
        End Get
    End Property
#End Region

#Region "Methods"
    Private Sub SetHandled(ByVal value As Boolean)
        Me._Handled = value
        RaiseEvent HandledChanged(Me, EventArgs.Empty)
    End Sub

    Public Sub AddHandlers() Implements IEventBinder.AddHandlers
        If Me.BindingSource Is Nothing Then Throw New NullReferenceException("BindingSource")
        For Each Binding In Me.BindingSource.GetBindings
            If Not Binding.CanBind Then Continue For
            Binding.BindHandler()
        Next
        Me.SetHandled(True)
    End Sub
    Public Sub RemoveHandlers() Implements IEventBinder.RemoveHandlers
        If Me.BindingSource Is Nothing Then Throw New NullReferenceException("BindingSource")
        For Each Binding In Me.BindingSource.GetBindings
            If Not Binding.CanBind Then Continue For
            Binding.UnbindHandler()
        Next
        Me.SetHandled(False)
    End Sub

    Public Sub BeginInit() Implements System.ComponentModel.ISupportInitialize.BeginInit
    End Sub
    Public Sub EndInit() Implements System.ComponentModel.ISupportInitialize.EndInit
        If Me.DesignMode Then Exit Sub
        If Me.BindingMode <> EventBindingMode.Auto Then Exit Sub
        If Me.MustUseDataSourceEvent Then
            Me.ManageDataSource()
        Else
            Me.AddHandlers()
        End If
    End Sub

    Private Sub ManageDataSource()
        AddHandler DirectCast(Me.DataSource, BindingSource).DataSourceChanged, AddressOf OnDataSourceChanged
    End Sub
    Private Function MustUseDataSourceEvent() As Boolean
        Return Me.DataSource IsNot Nothing AndAlso TypeOf Me.DataSource Is BindingSource
    End Function
#End Region

#Region "Event handlers"
    Private Sub OnDataSourceChanged(ByVal sender As Object, ByVal e As EventArgs)
        If Me._Handled Then Me.RemoveHandlers()
        Me.AddHandlers()
    End Sub
#End Region

End Class
