﻿Imports EventBindings.Extensions

Public Class ObservableEventBindingSource
    Inherits EventBindingSource
    Implements IEventBindingsLifeObserverStore
    Implements ILifeObserver(Of HandlingData)
    Implements ILifeObserver(Of MediationData)


#Region "Fields"
    Private _HandlingObservers As New List(Of ILifeObserver(Of HandlingData))
    Private _MediationObservers As New List(Of ILifeObserver(Of MediationData))
#End Region

#Region "Constructors"
    Public Sub New()
        MyBase.New()
    End Sub
    Public Sub New(ByVal container As IContainer)
        MyBase.New(container)
    End Sub
#End Region

#Region "IEventBindingsLifePublisher Implementation"
    Public Sub RegisterCandidate(ByVal observer As Object) Implements IEventBindingsLifeObserverStore.RegisterCandidate
        If observer Is Nothing Then Exit Sub
        If TypeOf observer Is ILifeObserver(Of HandlingData) Then
            Me.RegisterHandlingObserver(DirectCast(observer, ILifeObserver(Of HandlingData)))
        End If
        If TypeOf observer Is ILifeObserver(Of MediationData) Then
            Me.RegisterMediationObserver(DirectCast(observer, ILifeObserver(Of MediationData)))
        End If
    End Sub
    Public Sub RegisterHandlingObserver(ByVal observer As ILifeObserver(Of HandlingData)) Implements IEventBindingsLifeObserverStore.RegisterHandlingObserver
        If observer Is Nothing Then Exit Sub
        Me._HandlingObservers.AddDistinct(observer)
    End Sub
    Public Sub RegisterMediationObserver(ByVal observer As ILifeObserver(Of MediationData)) Implements IEventBindingsLifeObserverStore.RegisterMediationObserver
        If observer Is Nothing Then Exit Sub
        Me._MediationObservers.AddDistinct(observer)
    End Sub

    Public Sub UnregisterCandidate(ByVal observer As Object) Implements IEventBindingsLifeObserverStore.UnregisterCandidate
        If observer Is Nothing Then Exit Sub
        If TypeOf observer Is ILifeObserver(Of HandlingData) Then
            Me.UnregisterHandlingObserver(DirectCast(observer, ILifeObserver(Of HandlingData)))
        End If
        If TypeOf observer Is ILifeObserver(Of MediationData) Then
            Me.UnregisterMediationObserver(DirectCast(observer, ILifeObserver(Of MediationData)))
        End If
    End Sub
    Public Sub UnregisterHandlingObserver(ByVal observer As ILifeObserver(Of HandlingData)) Implements IEventBindingsLifeObserverStore.UnregisterHandlingObserver
        If observer Is Nothing Then Exit Sub
        Me._HandlingObservers.Remove(observer)
    End Sub
    Public Sub UnregisterMediationObserver(ByVal observer As ILifeObserver(Of MediationData)) Implements IEventBindingsLifeObserverStore.UnregisterMediationObserver
        If observer Is Nothing Then Exit Sub
        Me._MediationObservers.Remove(observer)
    End Sub
#End Region

#Region "ILifeObserver(Of HandlingData) Implentation"
    Private Sub OnCreatingHandling(ByVal owner As EventBinding) Implements ILifeObserver(Of HandlingData).OnCreating
        Me._HandlingObservers.NotifyCreating(owner)
    End Sub
    Private Sub OnHandlingCreated(ByVal owner As EventBinding, ByVal subject As HandlingData) Implements ILifeObserver(Of HandlingData).OnCreated
        Me._HandlingObservers.NotifyCreated(owner, subject)
    End Sub
    Private Sub OnDisposingHandling(ByVal owner As EventBinding, ByVal subject As HandlingData) Implements ILifeObserver(Of HandlingData).OnDisposing
        Me._HandlingObservers.NotifyDisposing(owner, subject)
    End Sub
    Private Sub OnHandlingDisposed(ByVal owner As EventBinding) Implements ILifeObserver(Of HandlingData).OnDisposed
        Me._HandlingObservers.NotifyDisposed(owner)
    End Sub
#End Region

#Region "ILifeObserver(Of MediationData) Implementation"
    Private Sub OnCreatingMediation(ByVal owner As EventBinding) Implements ILifeObserver(Of MediationData).OnCreating
        Me._MediationObservers.NotifyCreating(owner)
    End Sub
    Private Sub OnMediationCreated(ByVal owner As EventBinding, ByVal subject As MediationData) Implements ILifeObserver(Of MediationData).OnCreated
        Me._MediationObservers.NotifyCreated(owner, subject)
    End Sub
    Private Sub OnDisposingMediation(ByVal owner As EventBinding, ByVal subject As MediationData) Implements ILifeObserver(Of MediationData).OnDisposing
        Me._MediationObservers.NotifyDisposing(owner, subject)
    End Sub
    Private Sub OnMediationDisposed(ByVal owner As EventBinding) Implements ILifeObserver(Of MediationData).OnDisposed
        Me._MediationObservers.NotifyDisposed(owner)
    End Sub
#End Region
    
End Class
